from dynamodb import logger, Config
import boto3
from botocore.config import Config as BConfig


class DynamoDB:
    def __init__(self):
        logger.debug('[DynamoDB]: Initiating DynamoDB Connection Class')
        self._connection_parameter = None
        self._resource = None
        self._config = BConfig(retries={'max_attempts': 3, 'mode': 'standard'})

    def set_connection_parameter(self, **kwargs):
        self._connection_parameter = {
            'region': Config.AWS['region'] if not kwargs.get('region') else kwargs.get('region'),
            "s3_key_id": Config.AWS['s3_key_id'] if not kwargs.get('s3_key_id') else kwargs.get('s3_key_id'),
            "s3_key_secret": Config.AWS['s3_key_secret'] if not kwargs.get('s3_key_secret')
            else kwargs.get('s3_key_secret'),
            "dynamo_table": Config.AWS['dynamo_table'] if not kwargs.get('dynamo_table')
            else kwargs.get('dynamo_table')
        }

    def get_connection_parameter(self):
        return self._connection_parameter

    @property
    def connection(self):
        if self._resource is None:
            self.connect()
        return self._resource

    def connect(self):
        if self._connection_parameter is None:
            self.set_connection_parameter()
        try:
            logger.debug('[DynamoDB]: Creating DynamoDB connection')
            dynamodb_resource = boto3.resource('dynamodb', region_name=self._connection_parameter['region'],
                                               aws_access_key_id=self._connection_parameter['s3_key_id'],
                                               aws_secret_access_key=self._connection_parameter['s3_key_secret'],
                                               config=self._config)

            # For testing connection only, because boto3 only return resource class not exactly a connection
            table_name = self._connection_parameter.get('dynamo_table')
            if table_name:
                table = dynamodb_resource.Table(table_name)
                if table.table_status == 'ACTIVE':  # this throws exception as expected
                    self._resource = dynamodb_resource
                    logger.info(f'[DynamoDB]: Connection Successful. Connection={self._resource}')
                else:
                    raise Exception('Unable to connect to table=connection_backend')
        except Exception as e:
            self._resource = None
            logger.error(f'[DynamoDB]: connection issue, conn={self._resource}', exc_info=True)
            raise Exception(f'[DynamoDB]: Connection Error with DynamoDB. Error={e}')



import logging
import os
import sys
from dotenv import load_dotenv

if not load_dotenv(override=False):
    print('Could not find any .env file. The module will depend on system env only')

from config import Config

logger = logging.getLogger(os.getenv('APP_NAME', __name__))

logger.setLevel(os.getenv('LOG_LEVEL', 'INFO'))

if not logger.hasHandlers():
    formatter = logging.Formatter('%(asctime)s-%(module)s-%(lineno)s::%(levelname)s:: %(message)s')
    ch = logging.StreamHandler(sys.stdout)
    ch.setLevel(os.getenv('LOG_LEVEL', 'INFO'))
    ch.setFormatter(formatter)
    logger.addHandler(ch)

import dynamo
import queries



from dynamodb.dynamo import DynamoDB
from boto3.dynamodb.conditions import Key


class DynamoQueries:
    def __init__(self, table_name):
        self.db = DynamoDB()
        self.table = self.db.connection.Table(table_name)

    def get_pk_context(self, pk, pk_value, table_name):
        response = self.table.query(KeyConditionExpression=Key(pk).eq(pk_value))
        model_data = response['Items']
        return model_data

    def get_index_context(self, index_key, index_value, index_name, table_value):
        response = self.table.query(IndexName=index_name, KeyConditionExpression=Key(index_key).eq(index_value))
        model_data = response['Items']
        return model_data



from setuptools import setup

setup(
    name='zfunds-services',
    packages=['dynamodb', 'mongodb', 's3'],
    package_dir={'dynamodb': 'dynamodb', 'mongodb': 'mongodb', 's3': 's3'},
    version='0.0.4',
    license='MIT',
    platforms='cross-platfom, platform-independent',
    description='ZFunds basic services',
    long_description='Dependencies: coming soon',
    author='Yogesh Yadav',
    author_email='yogesh@zfunds.in',
    url='https://github.com/',
    download_url='https://github.com/',
    keywords=['dynamodb', 'mongodb', 's3'],
    install_requires=[
        'python-dotenv==0.19.2', 'boto3==1.21.17'
    ],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 3.6',
    ],
)



import os
from dotenv import load_dotenv

load_dotenv(override=False)


class Config:
    APP_NAME = os.getenv('APP_NAME', 'DYNAMODB')

    # Environment
    ENVIRONMENT = os.getenv('ENVIRONMENT', 'PRODUCTION')
    REGION = os.getenv('REGION', 'IND')
    LANGUAGE = os.getenv('APP_LANGUAGE', 'EN')

    # Log
    LOG_LEVEL = os.getenv('LOG_LEVEL', 'INFO')

    # Security
    ZFUNDS_API_KEY = os.getenv('ZFUNDS_API_KEY', '')

    # AWS
    AWS = {
        's3_key_id': os.getenv('AWS_S3_KEY_ID'),
        's3_key_secret': os.getenv('AWS_S3_SECRET_KEY'),
        'region': os.getenv('AWS_S3_REGION'),
        'dynamo_table': os.getenv('AWS_DYNAMO_TABLE')
    }
